__author__ = "hugo.inzirillo"

from napoleontoolbox.napoleon_config_tools.parser import target, config, getter

__all__ = ['target', 'config', 'getter']






if __name__ == '__main__':
    end = True