__author__ = "hugo.inzirillo"

from npxlogger.logger_manager import log

__all__ = ['log']
__version__ = "0.0.1"

########################## EXEMPLE ######################


# from npxlogger import log
# log.info()
# log.warning()
# log.error()
# log.warn()
